@@ -3,14 +3,14 @@ use strict;
 use warnings;
 package Email::MIME;
 # ABSTRACT: easy MIME message handling
-$Email::MIME::VERSION = '1.943'; # TRIAL
+$Email::MIME::VERSION = '1.947'; # TRIAL
 use Email::Simple 2.212; # nth header value
 use parent qw(Email::Simple);
 
 use Carp ();
 use Email::MessageID;
 use Email::MIME::Creator;
-use Email::MIME::ContentType 1.016; # type/subtype, not discrete/composite
+use Email::MIME::ContentType 1.022; # parse_content_disposition
 use Email::MIME::Encode;
 use Email::MIME::Encodings 1.314;
 use Email::MIME::Header;
@@ -18,6 +18,8 @@ use Email::MIME::Modifier;
 use Encode 1.9801 ();
 use Scalar::Util qw(reftype weaken);
 
+our @CARP_NOT = qw(Email::MIME::ContentType);
+
 #pod =head1 SYNOPSIS
 #pod
 #pod B<Wait!>  Before you read this, maybe you just need L<Email::Stuffer>, which is
@@ -123,7 +125,15 @@ my $NO_ENCODE_RE = qr/
 /ix;
 
 sub new {
-  my $self = shift->SUPER::new(@_);
+  my ($class, $text, $arg, @rest) = @_;
+  $arg ||= {};
+
+  my $encode_check = exists $arg->{encode_check}
+                   ? delete $arg->{encode_check}
+                   : Encode::FB_CROAK;
+
+  my $self = shift->SUPER::new($text, $arg, @rest);
+  $self->encode_check_set($encode_check);
   $self->{ct} = parse_content_type($self->content_type);
   $self->parts;
   return $self;
@@ -149,7 +159,8 @@ sub new {
 #pod an object with C<as_mime_string> method which implements conversion of that
 #pod object to MIME-encoded string.  That object method is called with two named
 #pod input parameters: C<charset> and C<header_name_length>.  It should return
-#pod MIME-encoded representation of the object.
+#pod MIME-encoded representation of the object.  As of 2017-07-25, the
+#pod header-value-as-object code is very young, and may yet change.
 #pod
 #pod In case header name is registered in C<%Email::MIME::Header::header_to_class_map>
 #pod hash then registered class is used for conversion from Unicode string to 8bit
@@ -162,7 +173,11 @@ sub new {
 #pod C<header_str>.  Its values will be used verbatim.
 #pod
 #pod C<attributes> is a hash of MIME attributes to assign to the part, and may
-#pod override portions of the header set in the C<header> parameter.
+#pod override portions of the header set in the C<header> parameter. The hash keys
+#pod correspond directly to methods for modifying a message from
+#pod C<Email::MIME::Modifier>. The allowed keys are: content_type, charset, name,
+#pod format, boundary, encoding, disposition, and filename. They will be mapped to
+#pod C<"$attr\_set"> for message modification.
 #pod
 #pod The C<parts> parameter is a list reference containing C<Email::MIME>
 #pod objects. Elements of the C<parts> list can also be a non-reference
@@ -177,12 +192,6 @@ sub new {
 #pod character string to be used as the body.  If you provide a C<body_str>
 #pod parameter, you B<must> provide C<charset> and C<encoding> attributes.
 #pod
-#pod Back to C<attributes>. The hash keys correspond directly to methods or
-#pod modifying a message from C<Email::MIME::Modifier>. The allowed keys are:
-#pod content_type, charset, name, format, boundary, encoding, disposition,
-#pod and filename. They will be mapped to C<"$attr\_set"> for message
-#pod modification.
-#pod
 #pod =cut
 
 my %CT_SETTER = map {; $_ => 1 } qw(
@@ -238,9 +247,15 @@ sub create {
     $CREATOR->_add_to_header(\$header, 'Content-Type' => 'text/plain',);
   }
 
-  my $email = $class->new($header);
+  my %pass_on;
 
-  for my $key (keys %attrs) {
+  if (exists $args{encode_check}) {
+    $pass_on{encode_check} = $args{encode_check};
+  }
+
+  my $email = $class->new($header, \%pass_on);
+
+  for my $key (sort keys %attrs) {
     $email->content_type_attribute_set($key => $attrs{$key});
   }
 
@@ -263,7 +278,7 @@ sub create {
     Carp::confess("body_str was given, but no encoding is defined")
       unless $attrs{encoding};
 
-    my $body_octets = Encode::encode($attrs{charset}, $args{body_str}, 1);
+    my $body_octets = Encode::encode($attrs{charset}, $args{body_str}, $email->encode_check);
     $email->body_set($body_octets);
   }
 
@@ -358,21 +373,29 @@ sub body_str {
     Carp::confess("can't get body as a string for " . $self->content_type);
   }
 
-  my $str = Encode::decode($encoding, $self->body, 1);
+  my $str = Encode::decode($encoding, $self->body, $self->encode_check);
   return $str;
 }
 
+our $MAX_DEPTH = 10;
+
 sub parts_multipart {
   my $self     = shift;
   my $boundary = $self->{ct}->{attributes}->{boundary};
 
+  our $DEPTH ||= 0;
+
+  Carp::croak("attempted to parse a MIME message more than $MAX_DEPTH deep")
+    if $MAX_DEPTH && $DEPTH > $MAX_DEPTH;
+
   # Take a message, join all its lines together.  Now try to Email::MIME->new
   # it with 1.861 or earlier.  Death!  It tries to recurse endlessly on the
   # body, because every time it splits on boundary it gets itself. Obviously
   # that means it's a bogus message, but a mangled result (or exception) is
   # better than endless recursion. -- rjbs, 2008-01-07
   return $self->parts_single_part
-    unless $boundary and $self->body_raw =~ /^--\Q$boundary\E\s*$/sm;
+    unless defined $boundary and length $boundary and
+           $self->body_raw =~ /^--\Q$boundary\E\s*$/sm;
 
   $self->{body_raw} = $self->SUPER::body;
 
@@ -395,7 +418,8 @@ sub parts_multipart {
   for my $bit (@bits) {
     $bit =~ s/\A[\n\r]+//smg;
     $bit =~ s/(?<!\x0d)$self->{mycrlf}\Z//sm;
-    my $email = (ref $self)->new($bit);
+    local $DEPTH = $DEPTH + 1;
+    my $email = (ref $self)->new($bit, { encode_check => $self->encode_check });
     push @parts, $email;
   }
 
@@ -425,10 +449,7 @@ sub filename {
   return $gcache{$self} if exists $gcache{$self};
 
   my $dis = $self->header("Content-Disposition") || '';
-  my $attrs
-    = $dis =~ s/^.*?;//
-    ? Email::MIME::ContentType::_parse_attributes($dis)
-    : {};
+  my $attrs = parse_content_disposition($dis)->{attributes};
   my $name = $attrs->{filename}
     || $self->{ct}{attributes}{name};
   return $name if $name or !$force;
@@ -529,7 +550,7 @@ sub boundary_set {
   my ($self, $value) = @_;
   my $ct_header = parse_content_type($self->header('Content-Type'));
 
-  if ($value) {
+  if (defined $value and length $value) {
     $ct_header->{attributes}->{boundary} = $value;
   } else {
     delete $ct_header->{attributes}->{boundary};
@@ -563,6 +584,42 @@ sub content_type_attribute_set {
   $self->_compose_content_type($ct_header);
 }
 
+#pod =method encode_check
+#pod
+#pod =method encode_check_set
+#pod
+#pod   $email->encode_check;
+#pod   $email->encode_check_set(0);
+#pod   $email->encode_check_set(Encode::FB_DEFAULT);
+#pod
+#pod Gets/sets the current C<encode_check> setting (default: I<FB_CROAK>).
+#pod This is the parameter passed to L<Encode/"decode"> and L<Encode/"encode">
+#pod when C<body_str()>, C<body_str_set()>, and C<create()> are called.
+#pod
+#pod With the default setting, Email::MIME may crash if the claimed charset
+#pod of a body does not match its contents (for example - utf8 data in a
+#pod text/plain; charset=us-ascii message).
+#pod
+#pod With an C<encode_check> of 0, the unrecognized bytes will instead be
+#pod replaced with the C<REPLACEMENT CHARACTER> (U+0FFFD), and may end up
+#pod as either that or question marks (?).
+#pod
+#pod See L<Encode/"Handling Malformed Data"> for more information.
+#pod
+#pod =cut
+
+sub encode_check {
+  my ($self) = @_;
+
+  return $self->{encode_check};
+}
+
+sub encode_check_set {
+  my ($self, $val) = @_;
+
+  return $self->{encode_check} = $val;
+}
+
 #pod =method encoding_set
 #pod
 #pod   $email->encoding_set( 'base64' );
@@ -644,7 +701,7 @@ sub body_str_set {
   Carp::confess("body_str was given, but no charset is defined")
     unless my $charset = $ct->{attributes}{charset};
 
-  my $body_octets = Encode::encode($charset, $body_str, 1);
+  my $body_octets = Encode::encode($charset, $body_str, $self->encode_check);
   $self->body_set($body_octets);
 }
 
@@ -681,9 +738,9 @@ sub filename_set {
   my $dis_header = $self->header('Content-Disposition');
   my ($disposition, $attrs);
   if ($dis_header) {
-    ($disposition) = ($dis_header =~ /^([^;]+)/);
-    $dis_header =~ s/^$disposition(?:;\s*)?//;
-    $attrs = Email::MIME::ContentType::_parse_attributes($dis_header) || {};
+    my $struct = parse_content_disposition($dis_header);
+    $disposition = $struct->{type};
+    $attrs = $struct->{attributes};
   }
   $filename ? $attrs->{filename} = $filename : delete $attrs->{filename};
   $disposition ||= 'inline';
@@ -716,7 +773,8 @@ sub parts_set {
   if (@{$parts} > 1 or $ct_header->{type} eq 'multipart') {
 
     # setup multipart
-    $ct_header->{attributes}->{boundary} ||= Email::MessageID->new->user;
+    $ct_header->{attributes}->{boundary} = Email::MessageID->new->user
+      unless defined $ct_header->{attributes}->{boundary} and length $ct_header->{attributes}->{boundary};
     my $bound = $ct_header->{attributes}->{boundary};
     foreach my $part (@{$parts}) {
       $body .= "$self->{mycrlf}--$bound$self->{mycrlf}";
@@ -887,7 +945,7 @@ Email::MIME - easy MIME message handling
 
 =head1 VERSION
 
-version 1.943
+version 1.947
 
 =head1 SYNOPSIS
 
@@ -1004,7 +1062,8 @@ be a text string that will be MIME-encoded as needed.  Alternatively it can be
 an object with C<as_mime_string> method which implements conversion of that
 object to MIME-encoded string.  That object method is called with two named
 input parameters: C<charset> and C<header_name_length>.  It should return
-MIME-encoded representation of the object.
+MIME-encoded representation of the object.  As of 2017-07-25, the
+header-value-as-object code is very young, and may yet change.
 
 In case header name is registered in C<%Email::MIME::Header::header_to_class_map>
 hash then registered class is used for conversion from Unicode string to 8bit
@@ -1017,7 +1076,11 @@ A similar C<header> parameter can be provided in addition to or instead of
 C<header_str>.  Its values will be used verbatim.
 
 C<attributes> is a hash of MIME attributes to assign to the part, and may
-override portions of the header set in the C<header> parameter.
+override portions of the header set in the C<header> parameter. The hash keys
+correspond directly to methods for modifying a message from
+C<Email::MIME::Modifier>. The allowed keys are: content_type, charset, name,
+format, boundary, encoding, disposition, and filename. They will be mapped to
+C<"$attr\_set"> for message modification.
 
 The C<parts> parameter is a list reference containing C<Email::MIME>
 objects. Elements of the C<parts> list can also be a non-reference
@@ -1032,12 +1095,6 @@ If C<body_str> is given instead of C<body> or C<parts>, it is assumed to be a
 character string to be used as the body.  If you provide a C<body_str>
 parameter, you B<must> provide C<charset> and C<encoding> attributes.
 
-Back to C<attributes>. The hash keys correspond directly to methods or
-modifying a message from C<Email::MIME::Modifier>. The allowed keys are:
-content_type, charset, name, format, boundary, encoding, disposition,
-and filename. They will be mapped to C<"$attr\_set"> for message
-modification.
-
 =head2 content_type_set
 
   $email->content_type_set( 'text/html' );
@@ -1062,6 +1119,28 @@ These four methods modify common C<Content-Type> attributes. If set to
 C<undef>, the attribute is removed. All other C<Content-Type> header
 information is preserved when modifying an attribute.
 
+=head2 encode_check
+
+=head2 encode_check_set
+
+  $email->encode_check;
+  $email->encode_check_set(0);
+  $email->encode_check_set(Encode::FB_DEFAULT);
+
+Gets/sets the current C<encode_check> setting (default: I<FB_CROAK>).
+This is the parameter passed to L<Encode/"decode"> and L<Encode/"encode">
+when C<body_str()>, C<body_str_set()>, and C<create()> are called.
+
+With the default setting, Email::MIME may crash if the claimed charset
+of a body does not match its contents (for example - utf8 data in a
+text/plain; charset=us-ascii message).
+
+With an C<encode_check> of 0, the unrecognized bytes will instead be
+replaced with the C<REPLACEMENT CHARACTER> (U+0FFFD), and may end up
+as either that or question marks (?).
+
+See L<Encode/"Handling Malformed Data"> for more information.
+
 =head2 encoding_set
 
   $email->encoding_set( 'base64' );
@@ -1264,6 +1343,12 @@ For example:
     + text/plain
     + text/html
 
+=head1 CONFIGURATION
+
+The variable C<$Email::MIME::MAX_DEPTH> is the maximum depth of parts that will
+be processed.  It defaults to 10, already higher than legitimate mail is ever
+likely to be.  This value may go up over time as the parser is improved.
+
 =head1 TODO
 
 All of the Email::MIME-specific guts should move to a single entry on the
@@ -1299,7 +1384,7 @@ Simon Cozens <simon@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Alex Vandiver Arthur Axel 'fREW' Schmidt Brian Cassidy David Steinbrunner Dotan Dimet Geraint Edwards Jesse Luehrs Kurt Anderson Lance A. Brown Matthew Horsfall (alh) memememomo Pali Shawn Sorichetti Tomohiro Hosaka
+=for stopwords Alex Vandiver Anirvan Chatterjee Arthur Axel 'fREW' Schmidt Brian Cassidy Dan Book David Steinbrunner Dotan Dimet dxdc Eric Wong Geraint Edwards Jesse Luehrs Kurt Anderson Lance A. Brown Matthew Horsfall (alh) memememomo Michael McClimon Pali Ricardo Signes Shawn Sorichetti Tomohiro Hosaka
 
 =over 4
 
@@ -1309,6 +1394,10 @@ Alex Vandiver <alexmv@mit.edu>
 
 =item *
 
+Anirvan Chatterjee <anirvan@users.noreply.github.com>
+
+=item *
+
 Arthur Axel 'fREW' Schmidt <frioux@gmail.com>
 
 =item *
@@ -1317,6 +1406,10 @@ Brian Cassidy <bricas@cpan.org>
 
 =item *
 
+Dan Book <grinnz@gmail.com>
+
+=item *
+
 David Steinbrunner <dsteinbrunner@pobox.com>
 
 =item *
@@ -1325,6 +1418,14 @@ Dotan Dimet <dotan@corky.net>
 
 =item *
 
+dxdc <dan@element26.net>
+
+=item *
+
+Eric Wong <e@80x24.org>
+
+=item *
+
 Geraint Edwards <gedge-oss@yadn.org>
 
 =item *
@@ -1349,10 +1450,18 @@ memememomo <memememomo@gmail.com>
 
 =item *
 
+Michael McClimon <michael@mcclimon.org>
+
+=item *
+
 Pali <pali@cpan.org>
 
 =item *
 
+Ricardo Signes <rjbs@semiotic.systems>
+
+=item *
+
 Shawn Sorichetti <ssoriche@coloredblocks.com>
 
 =item *
@@ -1492,6 +1601,12 @@ __END__
 #pod     + text/plain
 #pod     + text/html
 #pod
+#pod =head1 CONFIGURATION
+#pod
+#pod The variable C<$Email::MIME::MAX_DEPTH> is the maximum depth of parts that will
+#pod be processed.  It defaults to 10, already higher than legitimate mail is ever
+#pod likely to be.  This value may go up over time as the parser is improved.
+#pod
 #pod =head1 TODO
 #pod
 #pod All of the Email::MIME-specific guts should move to a single entry on the
